#
# spec file for package bpftrace
#
# Copyright (c) 2020 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#
# nodebuginfo


# Disable binary stripping because bpftrace depends on debug symbols in its
# implementation of the BEGIN trigger. See bsc#1178928.
%undefine _build_create_debug
%define __arch_install_post export NO_BRP_STRIP_DEBUG=true

Name:           bpftrace
Version:        0.11.4
Release:        lp152.2.5.1
Summary:        High-level tracing language for Linux eBPF
License:        Apache-2.0
Group:          Development/Tools/Debuggers
URL:            https://github.com/iovisor/bpftrace
Source:         https://github.com/iovisor/bpftrace/archive/v%{version}.tar.gz#/%{name}-%{version}.tar.gz
BuildRequires:  binutils
BuildRequires:  binutils-devel
BuildRequires:  bison
BuildRequires:  clang
BuildRequires:  clang-devel
BuildRequires:  cmake
BuildRequires:  flex
BuildRequires:  libbpf-devel
BuildRequires:  llvm-devel
BuildRequires:  pkgconfig
BuildRequires:  readline-devel
BuildRequires:  pkgconfig(libbcc) >= 0.11
BuildRequires:  pkgconfig(libelf)
BuildRequires:  pkgconfig(zlib)
ExcludeArch:    %arm %ix86

%description
High-level tracing language for Linux, allowing for instrumentation of
in-kernel and userspace state. It makes use of only upstream features such as
eBPF tracing (which builds on kprobes, uprobes, and a variety of other kernel
technologies). BPFtrace's language is inspired by awk and C, as well as other
tracers such as DTrace and SystemTap.

%package tools
Summary:        Example bpftrace scripts and other useful snippets
Group:          Development/Tools/Debuggers
Requires:       %{name} = %{version}
BuildArch:      noarch

%description tools
Collection of tools for quick instrumentation and inspection of a running
system. These are all BPFtrace scripts within %{_datadir}/bpftrace, and can be
easily modified to allow for different types of debugging.

%prep
%setup -q

# Correct the #!-line to avoid rpmlint warnings.
find tools -name '*.bt' -type f \
	-exec sed -i '1s|^#!%{_bindir}/env bpftrace|#!%{_bindir}/bpftrace|' '{}' ';'

%build
# Find libbfd.so and libopcodes.so. This is necessary because binutils gives
# these libraries very strange names which CMake cannot find. See boo#1162312.
LIBBFD="$(find "%{_libdir}" -type f -name 'libbfd*.so*')"
LIBOPCODES="$(find "%{_libdir}" -type f -name 'libopcodes*.so*')"

# We need to build with clang.
%define _lto_cflags %{nil}
export CC="clang"
export CXX="clang++"
%cmake \
	-DLIBBFD_LIBRARIES="${LIBBFD}" \
	-DLIBOPCODES_LIBRARIES="${LIBOPCODES}" \
	-DBUILD_SHARED_LIBS:BOOL=OFF \
	-DBUILD_STATIC_LIBS:BOOL=ON \
	-DBUILD_TESTING:BOOL=OFF
%cmake_build

%install
%cmake_install
# Set executable bit for tools.
chmod +x %{buildroot}%{_datadir}/bpftrace/tools/*.bt

%files
%{_bindir}/bpftrace
%{_mandir}/man8/bpftrace.8%{?ext_man}
%doc README.md docs/
%license LICENSE

%files tools
%dir %{_datadir}/bpftrace/
%dir %{_datadir}/bpftrace/tools
%{_datadir}/bpftrace/tools/*.bt
%dir %{_datadir}/bpftrace/tools/doc
%{_datadir}/bpftrace/tools/doc/*_example.txt
%{_mandir}/man8/*.8%{?ext_man}
%exclude %{_mandir}/man8/bpftrace.8%{ext_man}

%changelog
* Mon Nov 23 2020 Aleksa Sarai <asarai@suse.com>
- Do not strip bpftrace binaries -- bpftrace depends on debuginfos for
  /proc/self/exe in order to implement the BEGIN trigger. bsc#1178928
* Sat Nov 14 2020 Aleksa Sarai <asarai@suse.com>
- Update to bpftrace 0.11.4.
  * Alpine build bug fix release
  * bcc 0.17 support release
- Remove upstreamed patches:
  - detect-7-arg-bpf_attach_uprobe.patch
* Mon Nov  9 2020 Marcus Rueckert <mrueckert@suse.de>
- added detect-7-arg-bpf_attach_uprobe.patch:
  fix build with new bpf_attach_uprobe in TW
* Sat Oct 31 2020 Aleksa Sarai <asarai@suse.com>
- Update to bpftrace 0.11.2.
  * Support for LLVM 11.
* Wed Jul 15 2020 Aleksa Sarai <asarai@suse.com>
- Update to bpftrace 0.11.0.
* Tue Apr 14 2020 Ismail Dönmez <idonmez@suse.com>
- Update to version 0.10.0
  * Support for LLVM 9 and LLVM 10 has been added.
  * See the complete changelog at https://github.com/iovisor/bpftrace/releases/tag/v0.10.0
- Drop upstreamed patches:
  - boo1162312-cmake-use-_LIBRARIES-when-testing-for-libbfd-version.patch
  - install-man-dir.patch
* Wed Feb  5 2020 Aleksa Sarai <asarai@suse.com>
- Update to bpftrace 0.9.4.
  * New calls: signal, override, strncmp.
  * Support for attaching to `kprobes` at an offset.
  * Support for struct bitfields.
  See the upstream changelog for more details:
    <https://github.com/iovisor/bpftrace/releases/tag/v0.9.4>
* Fri Jan 31 2020 Aleksa Sarai <asarai@suse.com>
- Fix builds on Leap 15.1. This requires quite a bit of trickery related to
  binutils (see boo#1162312), but it's nothing too hard.
- Added patch to workaround boo#1162312.
  + boo1162312-cmake-use-_LIBRARIES-when-testing-for-libbfd-version.patch
* Thu Nov 21 2019 Luigi Baldoni <aloisio@gmx.com>
- Update to version 0.9.3
  * Allow attaching to uprobes at an offset
  * BTF support
  * integer casts
  * integer pointer casts
* Tue Nov 19 2019 Gary Ching-Pang Lin <glin@suse.com>
- Update bpftrace to v0.9.2
  + New environment variables (BPFTRACE_NO_USER_SYMBOLS,
    BPFTRACE_LOG_SIZE)
  + New probe type: memory watchpoint
  + Support for JSON output
- Exclude armv7 and i586 since bcc doesn't support USDT for these
  two architectures
* Tue Aug 20 2019 Martin Pluskal <mpluskal@suse.com>
- Cleanup using spec-cleaner
* Tue Aug 20 2019 Antonio Ojea <aojeagarcia@suse.com>
- Add readline missing dependency
* Sat Jul 13 2019 Aaron Puchert <aaronpuchert@alice-dsl.net>
- Use %%cmake for building, don't set flags manually.
- Use %%cmake_install instead of installing manually.
- install-man-dir.patch: Fix installation directory for man pages.
- Fix executable bit for tools.
- Fix shebangs in %%prep, use -exec instead of piping to xargs.
- Disable LTO, because -flto=<number> isn't supported by Clang.
  Also LTO is perhaps not so important here.
* Wed Jun 26 2019 Aleksa Sarai <asarai@suse.com>
- Update bpftrace to v0.9.1. Upstream changelog:
  * Additions:
    + Introduce compound assignment operators (+= and friends).
    + Add support for arrays and IPv6 for the ntop builtin function.
    + Add basic support to enums (treat them as constants).
    + Add macro definition support.
    + Add support for arrays and IPv6 for the ntop builtin function.
    + Allow comparison of two string variables.
    + Add pre and post behavior to ++ and -- operators.
  * Breaking changes:
  - Ban kprobes that cause CPU deadlocks.
    + Add unsafe-mode and make default execution mode safe-mode.
* Wed Apr 24 2019 Aleksa Sarai <asarai@suse.com>
- Update bpftrace to v0.9.
- Deprecated: sym() and stack() are deprecated.
    Use ksym() and kstack() instead.
* Tue Jan 15 2019 Aleksa Sarai <asarai@suse.com>
- Update bpftrace import to 0.0.0 (again).
- Remove patch for bcc-devel now that there is an upstream fix for it
  (https://github.com/iovisor/bpftrace/pull/327). boo#1117223
  - 0001-cmake-fix-builds-within-OBS.patch
* Sun Nov 25 2018 Aleksa Sarai <asarai@suse.com>
- Switch to using bcc-devel from Base:System. This required a few backports
  from upstream and some other various changes. boo#1117223
  https://github.com/iovisor/bpftrace/pull/227
  * 0001-cmake-fix-builds-within-OBS.patch
* Tue Oct 30 2018 Aleksa Sarai <asarai@suse.com>
- Initial bpftrace 0.0.0 import. Upstream doesn't really have versioning, so
  now we just use _service and pull from master. BCC required a bit of
  vendoring work, though hopefully we can switch to using bcc-tool-devel.
  boo#1117223
